@@ -1,14 +1,17 @@
-## SimInf, a framework for stochastic disease spread simulations
-## Copyright (C) 2015  Pavol Bauer
-## Copyright (C) 2015 - 2017  Stefan Engblom
-## Copyright (C) 2015 - 2017  Stefan Widgren
+## This file is part of SimInf, a framework for stochastic
+## disease spread simulations.
 ##
-## This program is free software: you can redistribute it and/or modify
+## Copyright (C) 2015 Pavol Bauer
+## Copyright (C) 2017 -- 2019 Robin Eriksson
+## Copyright (C) 2015 -- 2019 Stefan Engblom
+## Copyright (C) 2015 -- 2019 Stefan Widgren
+##
+## SimInf is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
 ##
-## This program is distributed in the hope that it will be useful,
+## SimInf is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
@@ -16,7 +19,12 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-library("SimInf")
+library(SimInf)
+library(tools)
+source("util/check.R")
+
+## Specify the number of threads to use.
+set_num_threads(1)
 
 ## For debugging
 sessionInfo()
@@ -31,27 +39,21 @@ sessionInfo()
 ## probability of becoming infected.
 ##
 ## At t = 1, two individuals are moved to node = 1. This should fail.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L),
-                class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 2,
-                         proportion = 1,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 2,
+                     proportion = 1,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:10,
@@ -74,14 +76,14 @@ model <- SISe3(u0        = u0,
                end_t4    = 365,
                epsilon   = 0)
 
-res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Unable to sample individuals for event.",
-                      res[[1]]$message)) > 0)
+res <- assertError(run(model))
+check_error(res, "Unable to sample individuals for event.")
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Unable to sample individuals for event.",
-                          res[[1]]$message)) > 0)
+    set_num_threads(2)
+    res <- assertError(run(model))
+    set_num_threads(1)
+    check_error(res, "Unable to sample individuals for event.")
 }
 
 ## 2 Nodes
@@ -92,26 +94,21 @@ if (SimInf:::have_openmp()) {
 ## probability of becoming infected.
 ##
 ## At t = 1, -1 individuals are moved to node = 1. This should fail.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = -1,
-                         proportion = 1,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = -1,
+                     proportion = 1,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:10,
@@ -134,14 +131,14 @@ model <- SISe3(u0        = u0,
                end_t4    = 365,
                epsilon   = 0)
 
-res <- tools::assertError(run(model, threads = 1))
-stopifnot(length(grep("Unable to sample individuals for event.",
-                      res[[1]]$message)) > 0)
+res <- assertError(run(model))
+check_error(res, "Unable to sample individuals for event.")
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(run(model, threads = 2))
-    stopifnot(length(grep("Unable to sample individuals for event.",
-                          res[[1]]$message)) > 0)
+    set_num_threads(2)
+    res <- assertError(run(model))
+    set_num_threads(1)
+    check_error(res, "Unable to sample individuals for event.")
 }
 
 ## 2 Nodes
@@ -153,50 +150,44 @@ if (SimInf:::have_openmp()) {
 ##
 ## At t = 1, a proportion of 10 individuals are moved to node = 1.
 ## This should fail.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 0,
-                         proportion = 10,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 0,
+                     proportion = 10,
+                     select     = 4,
+                     shift      = 0)
 
 ## We should not be able to create model with prop = 10
-res <- tools::assertError(SISe3(u0        = u0,
-                                tspan     = 0:10,
-                                events    = events,
-                                phi       = rep(0, 2),
-                                upsilon_1 = 0,
-                                upsilon_2 = 0,
-                                upsilon_3 = 0,
-                                gamma_1   = 1,
-                                gamma_2   = 1,
-                                gamma_3   = 1,
-                                alpha     = 0,
-                                beta_t1   = 1,
-                                beta_t2   = 1,
-                                beta_t3   = 1,
-                                beta_t4   = 1,
-                                end_t1    = 91,
-                                end_t2    = 182,
-                                end_t3    = 273,
-                                end_t4    = 365,
-                                epsilon   = 0))
-stopifnot(length(grep("prop must be in the range 0 <= prop <= 1",
-                      res[[1]]$message)) > 0)
+res <- assertError(SISe3(u0        = u0,
+                         tspan     = 0:10,
+                         events    = events,
+                         phi       = rep(0, 2),
+                         upsilon_1 = 0,
+                         upsilon_2 = 0,
+                         upsilon_3 = 0,
+                         gamma_1   = 1,
+                         gamma_2   = 1,
+                         gamma_3   = 1,
+                         alpha     = 0,
+                         beta_t1   = 1,
+                         beta_t2   = 1,
+                         beta_t3   = 1,
+                         beta_t4   = 1,
+                         end_t1    = 91,
+                         end_t2    = 182,
+                         end_t3    = 273,
+                         end_t4    = 365,
+                         epsilon   = 0))
+check_error(res, "prop must be in the range 0 <= prop <= 1", FALSE)
 
 ## Replace proportion = 10 to proportion = 1
 events$proportion <- 1
@@ -225,14 +216,12 @@ model <- SISe3(u0        = u0,
 ## Replace proportion = 1 with proportion = 10
 model@events@proportion <- 10
 
-res <- tools::assertError(.Call("SISe3_run", model, 1L, NULL, PACKAGE = "SimInf"))
-stopifnot(length(grep("Unable to sample individuals for event.",
-                      res[[1]]$message)) > 0)
+res <- assertError(.Call(SimInf:::SISe3_run, model, 1L, NULL))
+check_error(res, "Unable to sample individuals for event.")
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(.Call("SISe3_run", model, 2L, NULL, PACKAGE = "SimInf"))
-    stopifnot(length(grep("Unable to sample individuals for event.",
-                          res[[1]]$message)) > 0)
+    res <- assertError(.Call(SimInf:::SISe3_run, model, 2L, NULL))
+    check_error(res, "Unable to sample individuals for event.")
 }
 
 ## 2 Nodes
@@ -244,50 +233,44 @@ if (SimInf:::have_openmp()) {
 ##
 ## At t = 1, a proportion of -1 individuals are moved to node = 1.
 ## This should fail.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 0,
-                         proportion = -1,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 0,
+                     proportion = -1,
+                     select     = 4,
+                     shift      = 0)
 
 ## We should not be able to create model with proportion = -1
-res <- tools::assertError(SISe3(u0        = u0,
-                                tspan     = 0:10,
-                                events    = events,
-                                phi       = rep(0, 2),
-                                upsilon_1 = 0,
-                                upsilon_2 = 0,
-                                upsilon_3 = 0,
-                                gamma_1   = 1,
-                                gamma_2   = 1,
-                                gamma_3   = 1,
-                                alpha     = 0,
-                                beta_t1   = 1,
-                                beta_t2   = 1,
-                                beta_t3   = 1,
-                                beta_t4   = 1,
-                                end_t1    = 91,
-                                end_t2    = 182,
-                                end_t3    = 273,
-                                end_t4    = 365,
-                                epsilon   = 0))
-stopifnot(length(grep("prop must be in the range 0 <= prop <= 1",
-                      res[[1]]$message)) > 0)
+res <- assertError(SISe3(u0        = u0,
+                         tspan     = 0:10,
+                         events    = events,
+                         phi       = rep(0, 2),
+                         upsilon_1 = 0,
+                         upsilon_2 = 0,
+                         upsilon_3 = 0,
+                         gamma_1   = 1,
+                         gamma_2   = 1,
+                         gamma_3   = 1,
+                         alpha     = 0,
+                         beta_t1   = 1,
+                         beta_t2   = 1,
+                         beta_t3   = 1,
+                         beta_t4   = 1,
+                         end_t1    = 91,
+                         end_t2    = 182,
+                         end_t3    = 273,
+                         end_t4    = 365,
+                         epsilon   = 0))
+check_error(res, "prop must be in the range 0 <= prop <= 1", FALSE)
 
 ## Replace proportion = -1 with proportion = 0
 events$proportion <- 0
@@ -316,14 +299,12 @@ model <- SISe3(u0        = u0,
 ## Replace proportion = 0 with proportion = -1
 model@events@proportion <- -1
 
-res <- tools::assertError(.Call("SISe3_run", model, 1L, NULL, PACKAGE = "SimInf"))
-stopifnot(length(grep("Unable to sample individuals for event.",
-                      res[[1]]$message)) > 0)
+res <- assertError(.Call(SimInf:::SISe3_run, model, 1L, NULL))
+check_error(res, "Unable to sample individuals for event.")
 
 if (SimInf:::have_openmp()) {
-    res <- tools::assertError(.Call("SISe3_run", model, 2L, NULL, PACKAGE = "SimInf"))
-    stopifnot(length(grep("Unable to sample individuals for event.",
-                          res[[1]]$message)) > 0)
+    res <- assertError(.Call(SimInf:::SISe3_run, model, 2L, NULL))
+    check_error(res, "Unable to sample individuals for event.")
 }
 
 ## 2 Nodes
@@ -334,26 +315,21 @@ if (SimInf:::have_openmp()) {
 ## probability of becoming infected.
 ##
 ## At t = 1, a proportion of 0 individuals are moved to node = 1.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3  = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3  = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 0,
-                         proportion = 0,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 0,
+                     proportion = 0,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:2,
@@ -379,12 +355,9 @@ model <- SISe3(u0        = u0,
 U_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
                           0L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
                           0L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L),
-                        .Dim = c(12L, 3L),
-                        .Dimnames = list(c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3",
-                                           "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                                         c("0", "1", "2")))
+                        .Dim = c(12L, 3L))
 
-result <- run(model, threads = 1)
+result <- run(model)
 stopifnot(identical(model@G, result@G))
 stopifnot(identical(model@S, result@S))
 stopifnot(identical(trajectory(result, as.is = TRUE), U_expected))
@@ -394,7 +367,9 @@ stopifnot(identical(model@u0, result@u0))
 stopifnot(identical(model@events, result@events))
 
 if (SimInf:::have_openmp()) {
-    result_omp <- run(model, threads = 2)
+    set_num_threads(2)
+    result_omp <- run(model)
+    set_num_threads(1)
     stopifnot(identical(model@G, result_omp@G))
     stopifnot(identical(model@S, result_omp@S))
     stopifnot(identical(trajectory(result_omp, as.is = TRUE), U_expected))
@@ -412,26 +387,21 @@ if (SimInf:::have_openmp()) {
 ## probability of becoming infected.
 ##
 ## At t = 1, proportion of all (1) individuals are moved to node = 1.
-u0 <- structure(list(S_1 = c(0, 1),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3  = c(0, 0),
-                     I_3  = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 1),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3  = c(0, 0),
+                 I_3  = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 1,
-                         proportion = 0,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 1,
+                     proportion = 0,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:2,
@@ -457,12 +427,9 @@ model <- SISe3(u0        = u0,
 U_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
                           1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
                           1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L),
-                        .Dim = c(12L, 3L),
-                        .Dimnames = list(c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3",
-                                           "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                                         c("0", "1", "2")))
+                        .Dim = c(12L, 3L))
 
-result <- run(model, threads = 1)
+result <- run(model)
 stopifnot(identical(model@G, result@G))
 stopifnot(identical(model@S, result@S))
 stopifnot(identical(trajectory(result, as.is = TRUE), U_expected))
@@ -472,7 +439,9 @@ stopifnot(identical(model@u0, result@u0))
 stopifnot(identical(model@events, result@events))
 
 if (SimInf:::have_openmp()) {
-    result_omp <- run(model, threads = 2)
+    set_num_threads(2)
+    result_omp <- run(model)
+    set_num_threads(1)
     stopifnot(identical(model@G, result_omp@G))
     stopifnot(identical(model@S, result_omp@S))
     stopifnot(identical(trajectory(result_omp, as.is = TRUE), U_expected))
@@ -490,26 +459,21 @@ if (SimInf:::have_openmp()) {
 ## probability of becoming infected.
 ##
 ## At t = 1, Nkind = 1, one individual is moved to node = 1.
-u0 <- structure(list(S_1 = c(0, 2),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3  = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 2),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3  = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 1,
-                         proportion = 0,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 1,
+                     proportion = 0,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:2,
@@ -535,12 +499,9 @@ model <- SISe3(u0        = u0,
 U_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 2L, 0L, 0L, 0L, 0L, 0L,
                           1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L,
                           1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L),
-                        .Dim = c(12L, 3L),
-                        .Dimnames = list(c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3",
-                                           "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                                         c("0", "1", "2")))
+                        .Dim = c(12L, 3L))
 
-result <- run(model, threads = 1)
+result <- run(model)
 stopifnot(identical(model@G, result@G))
 stopifnot(identical(model@S, result@S))
 stopifnot(identical(trajectory(result, as.is = TRUE), U_expected))
@@ -550,7 +511,9 @@ stopifnot(identical(model@u0, result@u0))
 stopifnot(identical(model@events, result@events))
 
 if (SimInf:::have_openmp()) {
-    result_omp <- run(model, threads = 2)
+    set_num_threads(2)
+    result_omp <- run(model)
+    set_num_threads(1)
     stopifnot(identical(model@G, result_omp@G))
     stopifnot(identical(model@S, result_omp@S))
     stopifnot(identical(trajectory(result_omp, as.is = TRUE), U_expected))
@@ -569,26 +532,21 @@ if (SimInf:::have_openmp()) {
 ## infected.
 ##
 ## At t = 1, Nkind = 2, one individual is moved to node = 1.
-u0 <- structure(list(S_1 = c(0, 2),
-                     I_1 = c(0, 8),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L), class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 2),
+                 I_1 = c(0, 8),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 1,
-                         proportion = 0,
-                         select     = 4,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 1,
+                     proportion = 0,
+                     select     = 4,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:2,
@@ -614,13 +572,10 @@ model <- SISe3(u0        = u0,
 U_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 2L, 8L, 0L, 0L, 0L, 0L,
                           0L, 1L, 0L, 0L, 0L, 0L, 2L, 7L, 0L, 0L, 0L, 0L,
                           0L, 1L, 0L, 0L, 0L, 0L, 2L, 7L, 0L, 0L, 0L, 0L),
-                        .Dim = c(12L, 3L),
-                        .Dimnames = list(c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3",
-                                           "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                                         c("0", "1", "2")))
+                        .Dim = c(12L, 3L))
 
 set.seed(1)
-result <- run(model, threads = 1)
+result <- run(model)
 stopifnot(identical(model@G, result@G))
 stopifnot(identical(model@S, result@S))
 stopifnot(identical(trajectory(result, as.is = TRUE), U_expected))
@@ -631,7 +586,9 @@ stopifnot(identical(model@events, result@events))
 
 if (SimInf:::have_openmp()) {
     set.seed(1)
-    result_omp <- run(model, threads = 2)
+    set_num_threads(2)
+    result_omp <- run(model)
+    set_num_threads(1)
     stopifnot(identical(model@G, result_omp@G))
     stopifnot(identical(model@S, result_omp@S))
     stopifnot(identical(trajectory(result_omp, as.is = TRUE), U_expected))
@@ -650,27 +607,21 @@ if (SimInf:::have_openmp()) {
 ##
 ## At t = 1, one individual is moved to node = 1 using a select matrix
 ## that can only select a susceptible from S_1.
-u0 <- structure(list(S_1 = c(0, 2),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 0),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 0),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L),
-                class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 2),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 0),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 0),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 1,
-                         proportion = 0,
-                         select     = 1,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 1,
+                     proportion = 0,
+                     select     = 1,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:10,
@@ -703,16 +654,15 @@ U <- structure(
       1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L,
       0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L,
       0L, 0L, 0L, 0L),
-    .Dim = c(12L, 11L),
-    .Dimnames = list(c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3",
-                       "S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                     c("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10")))
+    .Dim = c(12L, 11L))
 
-res <- run(model, threads = 1)
+res <- run(model)
 stopifnot(identical(res@U, U))
 
 if (SimInf:::have_openmp()) {
-    res <- run(model, threads = 2)
+    set_num_threads(2)
+    res <- run(model)
+    set_num_threads(1)
     stopifnot(identical(res@U, U))
 }
 
@@ -725,27 +675,21 @@ if (SimInf:::have_openmp()) {
 ##
 ## At t = 1, three individuals are moved to node = 1 using a select
 ## matrix that can select from the susceptible S_1, S_2 and S_3.
-u0 <- structure(list(S_1 = c(0, 2),
-                     I_1 = c(0, 0),
-                     S_2 = c(0, 2),
-                     I_2 = c(0, 0),
-                     S_3 = c(0, 2),
-                     I_3 = c(0, 0)),
-                .Names = c("S_1", "I_1", "S_2", "I_2", "S_3", "I_3"),
-                row.names = c(NA, -2L),
-                class = "data.frame")
+u0 <- data.frame(S_1 = c(0, 2),
+                 I_1 = c(0, 0),
+                 S_2 = c(0, 2),
+                 I_2 = c(0, 0),
+                 S_3 = c(0, 2),
+                 I_3 = c(0, 0))
 
-events <- structure(list(event      = 3,
-                         time       = 1,
-                         node       = 2,
-                         dest       = 1,
-                         n          = 3,
-                         proportion = 0,
-                         select     = 1,
-                         shift      = 0),
-                    .Names = c("event", "time", "node", "dest",
-                        "n", "proportion", "select", "shift"),
-                    row.names = c(NA, -1L), class = "data.frame")
+events <- data.frame(event      = 3,
+                     time       = 1,
+                     node       = 2,
+                     dest       = 1,
+                     n          = 3,
+                     proportion = 0,
+                     select     = 1,
+                     shift      = 0)
 
 model <- SISe3(u0        = u0,
                tspan     = 0:10,
@@ -772,16 +716,18 @@ model <- SISe3(u0        = u0,
 model@events@E[3, 1] <- 1
 model@events@E[5, 1] <- 1
 
-S_expected <- structure(c(0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3),
-                        .Names = c("0", "1", "2", "3", "4",
-                                   "5", "6", "7", "8", "9", "10"))
+S_expected <- c(0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3)
 
-res <- run(model, threads = 1)
-S_observed <- colSums(trajectory(res, compartments = c("S_1", "S_2", "S_3"), node = 1, as.is = TRUE))
+res <- run(model)
+S_observed <- colSums(trajectory(res, compartments = c("S_1", "S_2", "S_3"),
+                                 node = 1, as.is = TRUE))
 stopifnot(identical(S_observed, S_expected))
 
 if (SimInf:::have_openmp()) {
-    res <- run(model, threads = 2)
-    S_observed <- colSums(trajectory(res, compartments = c("S_1", "S_2", "S_3"), node = 1, as.is = TRUE))
+    set_num_threads(2)
+    res <- run(model)
+    set_num_threads(1)
+    S_observed <- colSums(trajectory(res, compartments = c("S_1", "S_2", "S_3"),
+                                     node = 1, as.is = TRUE))
     stopifnot(identical(S_observed, S_expected))
 }
